package im

import (
	"applet/app/e"
	md "applet/app/md/im"
	svc "applet/app/svc/im"
	"github.com/gin-gonic/gin"
)

// PageEmoticon
// @Summary      Im-表情包(列表)
// @Tags         表情包
// @Description  表情包(列表)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @param req body  md.PageEmoticonReq true "相关参数"
// @Success      200   {object}  	md.GetBasicResp "设置列表"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/im/pageEmoticon [POST]
func PageEmoticon(c *gin.Context) {
	var req *md.PageEmoticonReq
	if err1 := c.ShouldBindJSON(&req); err1 != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err1.Error())
		return
	}
	err, resp := svc.PageEmoticon(*req)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
		return
	}
	e.OutSuc(c, resp, nil)
}

// AddEmoticon
// @Summary      Im-表情包(新增)
// @Tags         表情包
// @Description  表情包(新增)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @param req body  md.AddEmoticonReq true "相关参数"
// @Success      200   {string}  	 "success"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/im/addEmoticon [post]
func AddEmoticon(c *gin.Context) {
	var req *md.AddEmoticonReq
	if err1 := c.ShouldBindJSON(&req); err1 != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err1.Error())
		return
	}

	err := svc.AddEmoticon(*req)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
		return
	}
	e.OutSuc(c, "success", nil)
}

// SetEmoticonState
// @Summary      Im-表情包(更新状态)
// @Tags         表情包
// @Description  表情包(新增)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @param req body  md.SetEmoticonStateReq true "相关参数"
// @Success      200   {string}  	 "success"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/im/setEmoticonState [post]
func SetEmoticonState(c *gin.Context) {
	var req *md.SetEmoticonStateReq
	if err1 := c.ShouldBindJSON(&req); err1 != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err1.Error())
		return
	}

	err := svc.SetEmoticonState(*req)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
		return
	}
	e.OutSuc(c, "success", nil)
}

// UpdateEmoticon
// @Summary      Im-表情包(编辑)
// @Tags         表情包
// @Description  表情包(编辑)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @param req body  md.UpdateEmoticonReq true "相关参数"
// @Success      200   {string}  	 "success"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/im/updateEmoticon [post]
func UpdateEmoticon(c *gin.Context) {
	var req *md.UpdateEmoticonReq
	if err1 := c.ShouldBindJSON(&req); err1 != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err1.Error())
		return
	}

	err := svc.UpdateEmoticon(*req)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
		return
	}
	e.OutSuc(c, "success", nil)
}

// DeleteEmoticon
// @Summary      Im-表情包(删除)
// @Tags         表情包
// @Description  表情包(删除)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @param req body  md.DeleteEmoticonReq true "相关参数"
// @Success      200   {string}  	 "success"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/im/deleteEmoticon [post]
func DeleteEmoticon(c *gin.Context) {
	var req *md.DeleteEmoticonReq
	if err1 := c.ShouldBindJSON(&req); err1 != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err1.Error())
		return
	}

	err := svc.DeleteEmoticon(*req)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
		return
	}
	e.OutSuc(c, "success", nil)
}
