package egg_energy

import (
	"applet/app/db"
	"applet/app/e"
	md "applet/app/md/institutional_management/egg_energy"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/implement"
	"code.fnuoos.com/EggPlanet/egg_system_rules.git/rule/egg_energy/enum"
	"github.com/gin-gonic/gin"
)

// DynamicDataFlowList
// @Summary      制度中心-动态数据-动态数据流水(获取)
// @Tags         公排管理
// @Description  动态数据流水(获取)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Param    req    body	md.DynamicDataFlowListReq 	true  "页数、每页大小必传 起止时间、持有类型选传"
// @Success      200   {object}  md.DynamicDataFlowListResp	"成功返回"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/institutionalManagement/eggEnergy/availableEnergy/list [post]
func DynamicDataFlowList(c *gin.Context) {
	var req *md.DynamicDataFlowListReq
	if err1 := c.ShouldBindJSON(&req); err1 != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err1.Error())
		return
	}

	var kindList = []map[string]interface{}{
		{
			"name":  enum.EggEnergyAvailableEnergyFlowKind.String(enum.PersonalActivePointRedemption),
			"value": enum.PersonalActivePointRedemption,
		},
		{
			"name":  enum.EggEnergyAvailableEnergyFlowKind.String(enum.TeamActivePointRedemption),
			"value": enum.TeamActivePointRedemption,
		},
		{
			"name":  enum.EggEnergyAvailableEnergyFlowKind.String(enum.AccountBalanceExchangeEggEnergy),
			"value": enum.AccountBalanceExchangeEggEnergy,
		},
		{
			"name":  enum.EggEnergyAvailableEnergyFlowKind.String(enum.EggEnergyExchangeAccountBalance),
			"value": enum.EggEnergyExchangeAccountBalance,
		},
		{
			"name":  enum.EggEnergyAvailableEnergyFlowKind.String(enum.SettlementStarLevelDividends),
			"value": enum.SettlementStarLevelDividends,
		},
		{
			"name":  enum.EggEnergyAvailableEnergyFlowKind.String(enum.CommunityDividends),
			"value": enum.CommunityDividends,
		},
		{
			"name":  enum.EggEnergyAvailableEnergyFlowKind.String(enum.PlatformProfitability),
			"value": enum.PlatformProfitability,
		},
		{
			"name":  enum.EggEnergyAvailableEnergyFlowKind.String(enum.MarketplaceMerchantNumsAutoExchangeMarketplaceMerchantFunds),
			"value": enum.MarketplaceMerchantNumsAutoExchangeMarketplaceMerchantFunds,
		},
		{
			"name":  enum.EggEnergyAvailableEnergyFlowKind.String(enum.MarketplaceMerchantFundsAutoExchangeMarketplaceMerchantNums),
			"value": enum.MarketplaceMerchantFundsAutoExchangeMarketplaceMerchantNums,
		},
	}

	energyFlowDb := implement.NewEggEnergyAvailableEnergyFlowDb(db.Db)
	flows, total, err := energyFlowDb.EggEnergyAvailableEnergyFlowFindAndCount(req.Kind, req.Page, req.PageSize, req.StartAt, req.EndAt)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	list := make([]md.EggEnergyAvailableEnergyFlow, len(flows))
	for i, flow := range flows {
		list[i] = md.EggEnergyAvailableEnergyFlow{
			Id:                                  flow.Id,
			Kind:                                flow.Kind,
			Title:                               flow.Title,
			BeforePrice:                         flow.BeforePrice,
			AfterPrice:                          flow.AfterPrice,
			Amount:                              flow.Amount,
			BeforeNowEnergyTotalNums:            flow.BeforeNowEnergyTotalNums,
			AfterNowEnergyTotalNums:             flow.AfterNowEnergyTotalNums,
			AfterPlanetTotalValue:               flow.AfterPlanetTotalValue,
			BeforePlanetTotalValue:              flow.BeforePlanetTotalValue,
			BeforeMarketplaceMerchantValues:     flow.BeforeMarketplaceMerchantValues,
			AfterMarketplaceMerchantValues:      flow.AfterMarketplaceMerchantValues,
			BeforeMarketplaceMerchantFundValues: flow.BeforeMarketplaceMerchantFundValues,
			AfterMarketplaceMerchantFundValues:  flow.AfterMarketplaceMerchantFundValues,
			BeforeDevelopmentCommitteeValues:    flow.BeforeDevelopmentCommitteeValues,
			AfterDevelopmentCommitteeValues:     flow.AfterDevelopmentCommitteeValues,
			BeforePublicWelfareAndCharityValues: flow.BeforePublicWelfareAndCharityValues,
			AfterPublicWelfareAndCharityValues:  flow.AfterPublicWelfareAndCharityValues,
			BeforeStarLevelDividendsValues:      flow.BeforeStarLevelDividendsValues,
			AfterStarLevelDividendsValues:       flow.AfterStarLevelDividendsValues,
			BeforeDestructionQuantityValues:     flow.BeforeDestructionQuantityValues,
			AfterDestructionQuantityValues:      flow.AfterDestructionQuantityValues,
			BeforeCommunityDividends:            flow.BeforeCommunityDividends,
			AfterCommunityDividends:             flow.AfterCommunityDividends,
			CreateTime:                          flow.CreateTime,
		}
	}

	resp := md.DynamicDataFlowListResp{
		KindList: kindList,
		List:     list,
		Paginate: md.Paginate{
			Limit: req.PageSize,
			Page:  req.Page,
			Total: total,
		},
	}

	e.OutSuc(c, resp, nil)
}
