package egg_energy

import (
	"applet/app/db"
	"applet/app/e"
	md "applet/app/md/institutional_management/egg_energy"
	"applet/app/utils"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/implement"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/model"
	"github.com/gin-gonic/gin"
)

// GetContributionValueBasicSetting
// @Summary      制度中心-蛋蛋能量-贡献值-基础设置(获取)
// @Tags         贡献值
// @Description  贡献值-基础设置(获取)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Success      200   {object}   md.GetContributionValueBasicSettingResp  	"具体数据"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/institutionalManagement/eggEnergy/contributionValue/basic [get]
func GetContributionValueBasicSetting(c *gin.Context) {
	settingDb := implement.NewContributionValueBasicSettingDb(db.Db)
	setting, err := settingDb.ContributionValueBasicSettingGetOne()
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, nil)
		return
	}
	if setting == nil {
		m := model.ContributionValueBasicSetting{
			PostingDynamicLiked: 0,
			PostingCommentLiked: 0,
			HandOutRedPackage:   0,
			PublishCreation:     0,
		}
		_, err := settingDb.ContributionValueBasicSettingInsert(&m)
		if err != nil {
			e.OutErr(c, e.ERR_DB_ORM, nil)
			return
		}
		setting, err = settingDb.ContributionValueBasicSettingGetOne()
		if err != nil {
			e.OutErr(c, e.ERR_DB_ORM, nil)
			return
		}
	}
	resp := md.GetContributionValueBasicSettingResp{
		Id:                  setting.Id,
		PostingDynamicLiked: setting.PostingDynamicLiked,
		PostingCommentLiked: setting.PostingCommentLiked,
		HandOutRedPackage:   setting.HandOutRedPackage,
		PublishCreation:     setting.PublishCreation,
	}

	e.OutSuc(c, resp, nil)
}

// UpdateContributionValueBasicSetting
// @Summary      制度中心-蛋蛋能量-贡献值-基础设置(更新)
// @Tags         贡献值
// @Description  贡献值-基础设置(更新)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Param    req    body	md.UpdateContributionValueBasicSettingReq 	true  "id 必填"
// @Success      200   {int}   	"修改数据条数"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/institutionalManagement/eggEnergy/contributionValue/updateBasic [POST]
func UpdateContributionValueBasicSetting(c *gin.Context) {
	var req *md.UpdateContributionValueBasicSettingReq
	if err1 := c.ShouldBindJSON(&req); err1 != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err1.Error())
		return
	}

	settingDb := implement.NewContributionValueBasicSettingDb(db.Db)
	m := model.ContributionValueBasicSetting{
		PostingDynamicLiked: utils.StrToInt(req.PostingDynamicLiked),
		PostingCommentLiked: utils.StrToInt(req.PostingCommentLiked),
		HandOutRedPackage:   utils.StrToInt(req.HandOutRedPackage),
		PublishCreation:     utils.StrToInt(req.PublishCreation),
	}
	forceColumns := []string{"publish_creation", "hand_out_red_package", "posting_comment_liked", "posting_dynamic_liked"}
	affected, err := settingDb.ContributionValueBasicSettingUpdate(req.Id, &m, forceColumns...)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err)
		return
	}
	e.OutSuc(c, affected, nil)
}
