package egg_energy

import (
	"applet/app/db"
	"applet/app/e"
	md "applet/app/md/institutional_management/egg_energy"
	"applet/app/svc/egg_energy"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/implement"
	md2 "code.fnuoos.com/EggPlanet/egg_system_rules.git/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_es.git/es"
	"github.com/gin-gonic/gin"
)

// UserEggIndex
// @Summary      制度中心-蛋蛋能量-蛋蛋分管理(着陆页数据)
// @Tags         蛋蛋能量
// @Description  蛋蛋分管理(着陆页数据)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @param req    body	md.UserEggIndexReq 	true  "请求参数"
// @Success      200   {object}  md.UserEggIndexResp	"成功返回"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/institutionalManagement/eggEnergy/eggPoint/statisticsUserEggIndex [POST]
func UserEggIndex(c *gin.Context) {
	var req *md.UserEggIndexReq
	if err1 := c.ShouldBindJSON(&req); err1 != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err1.Error())
		return
	}
	var resp md.UserEggIndexResp
	var err error

	var esIndexName string
	if req.Year == "" && req.Week == "" {
		esIndexName, _ = es.GetLatestIndexFromAlias(md2.EggEnergyUserEggScoreEsAlias)
		resp.Year, resp.Week = egg_energy.GetYearsAndWeekStr(esIndexName)
	} else {
		esIndexName = md2.EggEnergyUserEggScoreEsAlias + "_" + req.Year + req.Week
		resp.Year, resp.Week = req.Year, req.Week
	}

	//统计用户蛋蛋分范围
	resp.StatisticsUserEggScoreValueRange, err = egg_energy.StatisticsUserEggScoreValueRange(esIndexName)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
	}
	//统计用户蛋蛋分"评比类型"占比
	resp.StatisticsUserEggKindProportion, err = egg_energy.StatisticsUserEggKindProportion(esIndexName)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
	}
	//查询蛋蛋分系数权重
	eggEnergyUserEggIndexWeightDb := implement.NewEggEnergyUserEggIndexWeightDb(db.Db)
	resp.EggEnergyUserEggIndexWeight, _, _ = eggEnergyUserEggIndexWeightDb.EggEnergyUserEggIndexWeightGet()

	resp.YearAndWeekList, err = egg_energy.GetYearsAndWeeks()
	if err != nil {
		return
	}
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
	}
	e.OutSuc(c, resp, nil)
}

// StatisticsUserEggIndex
// @Summary      制度中心-蛋蛋能量-蛋蛋分管理(着陆页数据分析)
// @Tags         蛋蛋能量
// @Description  蛋蛋分管理(着陆页数据分析)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @param req    body	md.StatisticsEggPointReq 	true  "请求参数"
// @Success      200   {object}  md.StatisticsEggPointResp	"成功返回"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/institutionalManagement/eggEnergy/eggPoint/statisticsUserEggIndex [POST]
func StatisticsUserEggIndex(c *gin.Context) {
	var req *md.StatisticsEggPointReq
	if err1 := c.ShouldBindJSON(&req); err1 != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err1.Error())
		return
	}
	var resp md.StatisticsEggPointResp
	var err error

	var esIndexName string
	if req.Year == "" && req.Week == "" {
		esIndexName, _ = es.GetLatestIndexFromAlias(md2.EggEnergyUserEggScoreEsAlias)
		resp.Year, resp.Week = egg_energy.GetYearsAndWeekStr(esIndexName)
	} else {
		esIndexName = md2.EggEnergyUserEggScoreEsAlias + "_" + req.Year + req.Week
		resp.Year, resp.Week = req.Year, req.Week
	}

	//统计用户"ecpm"范围
	resp.StatisticsUserEggEcpmRange, err = egg_energy.StatisticsUserEggEcpmRange(esIndexName)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
	}
	//统计用户"拉新人数"范围
	resp.StatisticsUserEggInviteUserNumsRange, err = egg_energy.StatisticsUserEggInviteUserNumsRange(esIndexName)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
	}
	//统计用户"团队活跃次数"范围
	resp.StatisticsUserEggTeamActivityNumsRange, err = egg_energy.StatisticsUserEggTeamActivityNumsRange(esIndexName)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
	}
	//统计用户"签到次数"范围
	resp.StatisticsUserEggSignInNumsRange, err = egg_energy.StatisticsUserEggSignInNumsRange(esIndexName)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
	}
	//统计用户"发红包次数"范围
	resp.StatisticsUserEggSendRedPackageNumsRange, err = egg_energy.StatisticsUserEggSendRedPackageNumsRange(esIndexName)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
	}
	//统计用户"蛋蛋能量兑换余额数量"范围
	resp.StatisticsUserEggEggEnergyExchangeAccountBalanceRange, err = egg_energy.StatisticsUserEggEggEnergyExchangeAccountBalanceRange(esIndexName)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
	}
	//统计用户"蛋蛋能量兑换余额数量"范围
	resp.StatisticsUserEggAccountBalanceExchangeEggEnergyNumsRange, err = egg_energy.StatisticsUserEggAccountBalanceExchangeEggEnergyNumsRange(esIndexName)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
	}
	//统计用户"发朋友圈次数"范围
	resp.StatisticsUserEggSendCircleOfFriendNumsRange, err = egg_energy.StatisticsUserEggSendCircleOfFriendNumsRange(esIndexName)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
	}
	//统计用户"论坛评论次数"范围
	resp.StatisticsUserEggForumCommentsNumsRange, err = egg_energy.StatisticsUserEggForumCommentsNumsRange(esIndexName)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
	}
	//统计用户"学院学习次数"范围
	resp.StatisticsUserEggCollegeLearningNumsRange, err = egg_energy.StatisticsUserEggCollegeLearningNumsRange(esIndexName)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
	}
	//统计用户"违规次数"范围
	resp.StatisticsUserEggViolateNumsRange, err = egg_energy.StatisticsUserEggViolateNumsRange(esIndexName)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
	}
	//统计用户"浏览界面次数"范围
	resp.StatisticsUserEggBrowseInterfaceNumsRange, err = egg_energy.StatisticsUserEggBrowseInterfaceNumsRange(esIndexName)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
	}
	//统计用户"个人活跃积分增量值"范围
	resp.StatisticsUserEggPersonAddActivityValueRange, err = egg_energy.StatisticsUserEggPersonAddActivityValueRange(esIndexName)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
	}

	resp.YearAndWeekList, err = egg_energy.GetYearsAndWeeks()
	if err != nil {
		return
	}
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
	}
	e.OutSuc(c, resp, nil)
}

// UserEggFlow
// @Summary      制度中心-蛋蛋能量-蛋蛋分管理(用户数据列表)
// @Tags         蛋蛋能量
// @Description  蛋蛋分管理(用户数据列表)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @param req    body	md.UserEggFlowReq 	true  "请求参数"
// @Success      200   {object}  md.UserEggFlowReqResp	"成功返回"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router        /api/institutionalManagement/eggEnergy/eggPoint/userEggFlow [POST]
func UserEggFlow(c *gin.Context) {
	var req *md.UserEggFlowReq
	if err1 := c.ShouldBindJSON(&req); err1 != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err1.Error())
		return
	}
	var resp md.UserEggFlowReqResp
	var err error
	//评分类型(0:未知 1:人工 2:系统)
	scoreValueKindMap := []map[string]interface{}{
		{
			"name":  "未知",
			"value": 0,
		},
		{
			"name":  "人工",
			"value": 1,
		},
		{
			"name":  "系统",
			"value": 2,
		},
	}
	resp.ScoreValueKindMap = scoreValueKindMap

	var esIndexName string
	if req.Year == "" && req.Week == "" {
		esIndexName, _ = es.GetLatestIndexFromAlias(md2.EggEnergyUserEggScoreEsAlias)
		resp.Year, resp.Week = egg_energy.GetYearsAndWeekStr(esIndexName)
	} else {
		esIndexName = md2.EggEnergyUserEggScoreEsAlias + "_" + req.Year + req.Week
		resp.Year, resp.Week = req.Year, req.Week
	}

	resp.List, resp.Total, err = egg_energy.QueryElasticsearch(*req, esIndexName)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
		return
	}
	resp.Page = req.Page
	resp.PageSize = req.PageSize
	e.OutSuc(c, resp, nil)
}

// ManualScore
// @Summary      制度中心-蛋蛋能量-蛋蛋分管理(人工打分)
// @Tags         蛋蛋能量
// @Description  蛋蛋分管理(人工打分)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @param req    body	md.ManualScoreReq 	true  "请求参数"
// @Success      200   {string}  	success "成功返回"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/institutionalManagement/eggEnergy/eggPoint/manualScore [POST]
func ManualScore(c *gin.Context) {
	var req *md.ManualScoreReq
	if err1 := c.ShouldBindJSON(&req); err1 != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err1.Error())
		return
	}
	esIndexName, _ := es.GetLatestIndexFromAlias(md2.EggEnergyUserEggScoreEsAlias)
	doc, err := es.FirstDoc(esIndexName, req.IndexId)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
		return
	}
	if doc == nil {
		e.OutErr(c, e.ERR_NO_DATA, "文档记录为空")
		return
	}
	_, err = es.UpdateDoc(esIndexName, req.IndexId, map[string]interface{}{
		"score_value":      req.Score,
		"score_value_kind": 1,
	})
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
		return
	}
	e.OutSuc(c, "success", nil)
}
