package new_user_red_package

import (
	"applet/app/db"
	"applet/app/e"
	"applet/app/md/marketing_applications/new_user_red_package"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/implement"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/model"
	"github.com/gin-gonic/gin"
	"time"
)

// NewUserRedPackageGetBasic
// @Summary      制度中心-营销应用-新人红包设置(获取)
// @Tags         营销应用
// @Description  新人红包设置(获取)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Success      200   {object}   md.NewUserRedPackageGetBasicResp "具体数据"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/marketingApplications/newUserRedPackage/getBasic [get]
func NewUserRedPackageGetBasic(c *gin.Context) {
	redPackageDb := implement.NewNewUserRedPackageDb(db.Db)
	setting, err := redPackageDb.NewUserRedPackageGetOne()
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	if setting == nil {
		now := time.Now()
		setting = &model.NewUserRedPackage{
			IsOpen:      0,
			TotalAmount: "",
			Days:        0,
			IsDouble:    0,
			CreateAt:    now.Format("2006-01-02 15:04:05"),
			UpdateAt:    now.Format("2006-01-02 15:04:05"),
		}
		_, err := redPackageDb.NewUserRedPackageInsert(setting)
		if err != nil {
			e.OutErr(c, e.ERR_DB_ORM, err.Error())
			return
		}
	}

	resp := md.NewUserRedPackageGetBasicResp{
		IsOpen:      setting.IsOpen,
		TotalAmount: setting.TotalAmount,
		Days:        setting.Days,
		IsDouble:    setting.IsDouble,
		CreateAt:    setting.CreateAt,
		UpdateAt:    setting.UpdateAt,
	}

	e.OutSuc(c, resp, nil)
}

// NewUserRedPackageUpdateBasic
// @Summary      制度中心-营销应用-新人红包设置(修改)
// @Tags         营销应用
// @Description  新人红包设置(修改)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Param        req 	body      md.NewUserRedPackageUpdateBasicReq          true   "新人红包设置表单"
// @Success      200   {string}  				"success"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/marketingApplications/newUserRedPackage/updateBasic [put]
func NewUserRedPackageUpdateBasic(c *gin.Context) {
	var req *md.NewUserRedPackageUpdateBasicReq
	if err := c.ShouldBindJSON(&req); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}

	redPackageDb := implement.NewNewUserRedPackageDb(db.Db)
	setting, err := redPackageDb.NewUserRedPackageGetOne()
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}

	forceColumns := []string{"is_open", "total_amount", "days", "is_double"}

	newSetting := &model.NewUserRedPackage{
		Id:          setting.Id,
		IsOpen:      req.IsOpen,
		TotalAmount: req.TotalAmount,
		Days:        req.Days,
		IsDouble:    req.IsDouble,
	}

	_, err = redPackageDb.NewUserRedPackageUpdate(setting.Id, newSetting, forceColumns...)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}

	e.OutSuc(c, "success", nil)
}

// NewUserRedPackageRecordList
// @Summary      制度中心-营销应用-新人红包列表(查询)
// @Tags         营销应用
// @Description  新人红包列表(查询)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Param        req 	body      md.NewUserRedPackageRecordListReq          true   "新人红包列表查询条件(分页信息必填)"
// @Success      200   {object}  md.NewUserRedPackageRecordListResp			"具体数据"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/marketingApplications/newUserRedPackage/recordList [post]
func NewUserRedPackageRecordList(c *gin.Context) {
	var req *md.NewUserRedPackageRecordListReq
	if err := c.ShouldBindJSON(&req); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}

	recordsDb := implement.NewNewUserRedPackageWithUserRecordsDb(db.Db)
	records, total, err := recordsDb.NewUserRedPackageWithUserRecordsFind(req.Page, req.Limit, req.Uid, req.StartAt, req.EndAt)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}

	list := make([]md.NewUserRedPackageRecordNode, len(*records))
	for i, record := range *records {
		list[i].SystemID = record.Id
		list[i].Uid = record.Uid
		list[i].TotalAmount = record.TotalAmount
		list[i].Days = record.Days
		list[i].BalanceAmount = record.BalanceAmount
		list[i].BalanceDays = record.BalanceDays
		list[i].State = record.State
		list[i].CreateAt = record.CreateAt
		list[i].UpdateAt = record.UpdateAt
	}

	resp := md.NewUserRedPackageRecordListResp{
		List: list,
		Paginate: md.Paginate{
			Page:  req.Page,
			Limit: req.Limit,
			Total: total,
		},
	}

	e.OutSuc(c, resp, nil)
}

// NewUserRedPackageRecordFlowList
// @Summary      制度中心-营销应用-新人红包列表明细(查询)
// @Tags         营销应用
// @Description  新人红包列表明细(查询)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Param        req 	body      md.NewUserRedPackageRecordFlowListReq        true   "新人红包列表明细查询条件(分页信息、用户 ID必填)"
// @Success      200   {object}  md.NewUserRedPackageRecordFlowListResp			"具体数据"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/marketingApplications/newUserRedPackage/recordFlowList [post]
func NewUserRedPackageRecordFlowList(c *gin.Context) {
	var req *md.NewUserRedPackageRecordFlowListReq
	if err := c.ShouldBindJSON(&req); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}

	flowDb := implement.NewNewUserRedPackageWithUserRecordsFlowDb(db.Db)
	flows, total, err := flowDb.NewUserRedPackageWithUserRecordsFlowFindAndCount(req.Page, req.Limit, req.Uid, req.StartAt, req.EndAt)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	list := make([]md.NewUserRedPackageRecordFlowNode, len(*flows))
	for i, flow := range *flows {
		list[i].Id = flow.Id
		list[i].Uid = flow.Uid
		list[i].BasicAmount = flow.BasicAmount
		list[i].TotalAmount = flow.TotalAmount
		list[i].DayNum = flow.DayNum
		list[i].IsDouble = flow.IsDouble
		list[i].DoubleRate = flow.DoubleRate
		list[i].BalanceAmount = flow.BalanceAmount
		list[i].BalanceDays = flow.BalanceDays
		list[i].ReceiveDays = flow.ReceiveDays
		list[i].CreateAt = flow.CreateAt
	}

	resp := md.NewUserRedPackageRecordFlowListResp{
		List: list,
		Paginate: md.Paginate{
			Page:  req.Page,
			Limit: req.Limit,
			Total: total,
		},
	}
	e.OutSuc(c, resp, nil)
}
