package mw

import (
	"github.com/gin-gonic/gin"
)

// Cors 跨域
func Cors(c *gin.Context) {
	// 放行所有OPTIONS方法
	if c.Request.Method == "OPTIONS" {
		c.AbortWithStatus(204)
		return
	}

	origin := c.Request.Header.Get("Origin") // 请求头部
	if origin != "" {
		c.Header("Access-Control-Allow-Origin", origin)                                // 这是允许访问来源域
		c.Header("Access-Control-Allow-Methods", "POST,GET,OPTIONS,PUT,DELETE,UPDATE") // 服务器支持的所有跨域请求的方法,为了避免浏览次请求的多次'预检'请求
		// header的类型
		c.Header("Access-Control-Allow-Headers", "Authorization,Content-Length,X-CSRF-Token,Token,session,X_Requested_With,Accept,Origin,Host,Connection,Accept-Encoding,Accept-Language,DNT,X-CustomHeader,Keep-Alive,User-Agent,X-Requested-With,If-Modified-Since,Cache-Control,Content-Type,Pragma,X-Mx-ReqToken")
		// 允许跨域设置,可以返回其他子段
		// 跨域关键设置 让浏览器可以解析
		c.Header("Access-Control-Expose-Headers", "Content-Length,Access-Control-Allow-Origin,Access-Control-Allow-Headers,Cache-Control,Content-Language,Content-Type,Expires,Last-Modified,Pragma,FooBar")
		c.Header("Access-Control-Max-Age", "172800")          // 缓存请求信息 单位为秒
		c.Header("Access-Control-Allow-Credentials", "false") //  跨域请求是否需要带cookie信息 默认设置为true
		c.Set("Content-Type", "Application/json")             // 设置返回格式是json
	}
	c.Next()
}
